@@ -48,9 +48,6 @@ to reduce overhead accessing the real controls in kernel drivers.
 #include <string.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#ifndef DOC_HIDDEN
-#define __USE_GNU
-#endif
 #include "control_local.h"
 #ifdef HAVE_LIBPTHREAD
 #include <pthread.h>
@@ -234,7 +231,7 @@ static int snd_hctl_compare_mixer_priority_lookup(const char **name, const char
 
 static int get_compare_weight(const snd_ctl_elem_id_t *id)
 {
-	static const char *names[] = {
+	static const char *const names[] = {
 		"Master",
 		"Hardware Master",
 		"Headphone",
@@ -270,7 +267,7 @@ static int get_compare_weight(const snd_ctl_elem_id_t *id)
 		"IEC958",
 		NULL
 	};
-	static const char *names1[] = {
+	static const char *const names1[] = {
 		"Switch",
 		"Volume",
 		"Playback",
@@ -284,7 +281,7 @@ static int get_compare_weight(const snd_ctl_elem_id_t *id)
 		"-",
 		NULL
 	};
-	static const char *names2[] = {
+	static const char *const names2[] = {
 		"Switch",
 		"Volume",
 		"Bypass",
@@ -471,8 +468,9 @@ int snd_hctl_compare_fast(const snd_hctl_elem_t *c1,
 static int snd_hctl_compare_default(const snd_hctl_elem_t *c1,
 				    const snd_hctl_elem_t *c2)
 {
-	int res;
-	int d = c1->id.iface - c2->id.iface;
+	int res, d;
+
+	d = c1->id.iface - c2->id.iface;
 	if (d != 0)
 		return d;
 	if (c1->id.iface == SNDRV_CTL_ELEM_IFACE_MIXER) {
@@ -480,11 +478,16 @@ static int snd_hctl_compare_default(const snd_hctl_elem_t *c1,
 		if (d != 0)
 			return d;
 	}
+	d = c1->id.device - c2->id.device;
+	if (d != 0)
+		return d;
+	d = c1->id.subdevice - c2->id.subdevice;
+	if (d != 0)
+		return d;
 	res = strcmp((const char *)c1->id.name, (const char *)c2->id.name);
 	if (res != 0)
 		return res;
-	d = c1->id.index - c2->id.index;
-	return d;
+	return c1->id.index - c2->id.index;
 }
 
 /**
@@ -758,7 +761,6 @@ static int snd_hctl_handle_event(snd_hctl_t *hctl, snd_ctl_event_t *event)
 	if (event->data.elem.mask & (SNDRV_CTL_EVENT_MASK_VALUE |
 				     SNDRV_CTL_EVENT_MASK_INFO)) {
 		elem = snd_hctl_find_elem(hctl, &event->data.elem.id);
-		assert(elem);
 		if (!elem)
 			return -ENOENT;
 		res = snd_hctl_elem_throw_event(elem, event->data.elem.mask &
